#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains custom typing aliases for internal use within the library.

Warning:
    Contents of this module are intended to be used internally by the library and *not* by the
    user. Changes to this module are not considered breaking changes and may not be documented in
    the changelog.
"""

import datetime as dtm
from collections.abc import Callable, Collection
from pathlib import Path
from typing import IO, TYPE_CHECKING, Any, Literal, TypeAlias, TypeVar, Union

from telegram._utils.defaultvalue import DefaultValue

if TYPE_CHECKING:
    from telegram import (
        ForceReply,
        InlineKeyboardMarkup,
        InputFile,
        ReplyKeyboardMarkup,
        ReplyKeyboardRemove,
    )

# We guarantee that InputFile will be defined at runtime, so we can use a string here and ignore
# ruff.
# See https://github.com/python-telegram-bot/python-telegram-bot/pull/4827#issuecomment-2973060875
# on why we're doing this workaround.
# TODO: Use `type` syntax when we drop support for Python 3.11.
FileLike: TypeAlias = IO[bytes] | "InputFile"  # noqa: TC010
"""Either a bytes-stream (e.g. open file handler) or a :class:`telegram.InputFile`."""

FilePathInput: TypeAlias = str | Path
"""A filepath either as string or as :obj:`pathlib.Path` object."""

FileInput: TypeAlias = FilePathInput | FileLike | bytes | str
"""Valid input for passing files to Telegram. Either a file id as string, a file like object,
a local file path as string, :class:`pathlib.Path` or the file contents as :obj:`bytes`."""

JSONDict: TypeAlias = dict[str, Any]
"""Dictionary containing response from Telegram or data to send to the API."""

DVValueType = TypeVar("DVValueType")  # pylint: disable=invalid-name
DVType: TypeAlias = DVValueType | DefaultValue[DVValueType]
"""Generic type for a variable which can be either `type` or `DefaultValue[type]`."""
ODVInput: TypeAlias = DefaultValue[DVValueType] | DVValueType | DefaultValue[None] | None
"""Generic type for bot method parameters which can have defaults. ``ODVInput[type]`` is the same
as ``Union[DefaultValue[type], type, DefaultValue[None], None]``."""
DVInput: TypeAlias = DefaultValue[DVValueType] | DVValueType | DefaultValue[None]
"""Generic type for bot method parameters which can have defaults. ``DVInput[type]`` is the same
as ``Union[DefaultValue[type], type, DefaultValue[None]]``."""

RT = TypeVar("RT")
SCT: TypeAlias = RT | Collection[RT]  # pylint: disable=invalid-name
"""Single instance or collection of instances."""

# See comment above on why we're stuck using a Union here.
ReplyMarkup: TypeAlias = Union[
    "InlineKeyboardMarkup", "ReplyKeyboardMarkup", "ReplyKeyboardRemove", "ForceReply"
]
"""Type alias for reply markup objects.

.. versionadded:: 20.0
"""

FieldTuple: TypeAlias = tuple[str, bytes | IO[bytes], str]
"""Alias for return type of `InputFile.field_tuple`."""
UploadFileDict: TypeAlias = dict[str, FieldTuple]
"""Dictionary containing file data to be uploaded to the API."""

HTTPVersion: TypeAlias = Literal["1.1", "2.0", "2"]
"""Allowed HTTP versions.

.. versionadded:: 20.4"""

CorrectOptionID: TypeAlias = Literal[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]  # pylint: disable=invalid-name

MarkdownVersion: TypeAlias = Literal[1, 2]

SocketOpt: TypeAlias = (
    tuple[int, int, int] | tuple[int, int, bytes | bytearray] | tuple[int, int, None, int]
)

BaseUrl: TypeAlias = str | Callable[[str], str]

TimePeriod: TypeAlias = int | dtm.timedelta



import asyncio

import pytest

from telegram.ext import ApplicationBuilder, Updater
from telegram.ext.filters import MessageFilter, UpdateFilter
from tests.auxil.constants import PRIVATE_KEY
from tests.auxil.envvars import TEST_WITH_OPT_DEPS
from tests.auxil.monkeypatch import return_true
from tests.auxil.networking import OfflineRequest
from tests.auxil.pytest_classes import PytestApplication, PytestBot, make_bot

# This module overrides the bot fixtures defined in the global conftest.py to use the offline bot.
# We don't want the tests on telegram.ext to depend on the network, so we use the offline bot
# instead.


@pytest.fixture(scope="session")
async def bot(bot_info, offline_bot):
    return offline_bot


@pytest.fixture
async def app(bot_info, monkeypatch):
    # We build a new bot each time so that we use `app` in a context manager without problems
    application = (
        ApplicationBuilder()
        .bot(make_bot(bot_info, offline=True))
        .application_class(PytestApplication)
        .build()
    )
    monkeypatch.setattr(application.bot, "delete_webhook", return_true)
    monkeypatch.setattr(application.bot, "set_webhook", return_true)
    yield application
    if application.running:
        await application.stop()
        await application.shutdown()


@pytest.fixture
async def updater(bot_info, monkeypatch):
    # We build a new bot each time so that we use `updater` in a context manager without problems
    up = Updater(bot=make_bot(bot_info, offline=True), update_queue=asyncio.Queue())
    monkeypatch.setattr(up.bot, "delete_webhook", return_true)
    monkeypatch.setattr(up.bot, "set_webhook", return_true)
    yield up
    if up.running:
        await up.stop()
        await up.shutdown()


@pytest.fixture
def one_time_bot(bot_info):
    """A function scoped bot since the session bot would shutdown when `async with app` finishes"""
    return make_bot(bot_info, offline=True)


@pytest.fixture(scope="session")
async def cdc_bot(bot_info):
    """Makes an ExtBot instance with the given bot_info that uses arbitrary callback_data"""
    async with make_bot(bot_info, arbitrary_callback_data=True, offline=True) as _bot:
        yield _bot


@pytest.fixture(scope="session")
async def raw_bot(bot_info):
    """Makes an regular Bot instance with the given bot_info"""
    async with PytestBot(
        bot_info["token"],
        private_key=PRIVATE_KEY if TEST_WITH_OPT_DEPS else None,
        request=OfflineRequest(),
        get_updates_request=OfflineRequest(),
    ) as _bot:
        yield _bot


@pytest.fixture
async def one_time_raw_bot(bot_info):
    """Makes an regular Bot instance with the given bot_info"""
    return PytestBot(
        bot_info["token"],
        private_key=PRIVATE_KEY if TEST_WITH_OPT_DEPS else None,
        request=OfflineRequest(),
        get_updates_request=OfflineRequest(),
    )


@pytest.fixture(
    scope="class",
    params=[{"class": MessageFilter}, {"class": UpdateFilter}],
    ids=["MessageFilter", "UpdateFilter"],
)
def mock_filter(request):
    class MockFilter(request.param["class"]):
        def __init__(self):
            super().__init__()
            self.tested = False

        def filter(self, _):
            self.tested = True

    return MockFilter()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm
import inspect
from copy import deepcopy

import pytest

from telegram import (
    ChatMember,
    ChatMemberAdministrator,
    ChatMemberBanned,
    ChatMemberLeft,
    ChatMemberMember,
    ChatMemberOwner,
    ChatMemberRestricted,
    Dice,
    User,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.constants import ChatMemberStatus
from tests.auxil.slots import mro_slots


@pytest.fixture
def chat_member():
    return ChatMember(ChatMemberTestBase.user, ChatMemberTestBase.status)


class ChatMemberTestBase:
    status = ChatMemberStatus.MEMBER
    user = User(1, "test_user", is_bot=False)
    is_anonymous = True
    custom_title = "test_title"
    can_be_edited = True
    can_manage_chat = True
    can_delete_messages = True
    can_manage_video_chats = True
    can_restrict_members = True
    can_promote_members = True
    can_change_info = True
    can_invite_users = True
    can_post_messages = True
    can_edit_messages = True
    can_pin_messages = True
    can_post_stories = True
    can_edit_stories = True
    can_delete_stories = True
    can_manage_topics = True
    until_date = dtm.datetime.now(UTC).replace(microsecond=0)
    can_send_polls = True
    can_send_other_messages = True
    can_add_web_page_previews = True
    can_send_audios = True
    can_send_documents = True
    can_send_photos = True
    can_send_videos = True
    can_send_video_notes = True
    can_send_voice_notes = True
    can_send_messages = True
    is_member = True
    can_manage_direct_messages = True
    can_manage_tags = True
    can_edit_tag = True
    tag = "test_tag"


class TestChatMemberWithoutRequest(ChatMemberTestBase):
    def test_slot_behaviour(self, chat_member):
        inst = chat_member
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_status_enum_conversion(self, chat_member):
        assert type(ChatMember(ChatMemberTestBase.user, "member").status) is ChatMemberStatus
        assert ChatMember(ChatMemberTestBase.user, "unknown").status == "unknown"

    def test_de_json(self, offline_bot):
        data = {"status": "unknown", "user": self.user.to_dict()}
        chat_member = ChatMember.de_json(data, offline_bot)
        assert chat_member.api_kwargs == {}
        assert chat_member.status == "unknown"
        assert chat_member.user == self.user

    @pytest.mark.parametrize(
        ("status", "subclass"),
        [
            ("administrator", ChatMemberAdministrator),
            ("kicked", ChatMemberBanned),
            ("left", ChatMemberLeft),
            ("member", ChatMemberMember),
            ("creator", ChatMemberOwner),
            ("restricted", ChatMemberRestricted),
        ],
    )
    def test_de_json_subclass(self, offline_bot, status, subclass):
        json_dict = {
            "status": status,
            "user": self.user.to_dict(),
            "is_anonymous": self.is_anonymous,
            "is_member": self.is_member,
            "until_date": to_timestamp(self.until_date),
            **{name: value for name, value in inspect.getmembers(self) if name.startswith("can_")},
        }
        chat_member = ChatMember.de_json(json_dict, offline_bot)

        assert type(chat_member) is subclass
        assert set(chat_member.api_kwargs.keys()) == set(json_dict.keys()) - set(
            subclass.__slots__
        ) - {"status", "user"}
        assert chat_member.user == self.user

    def test_to_dict(self, chat_member):
        assert chat_member.to_dict() == {
            "status": chat_member.status,
            "user": chat_member.user.to_dict(),
        }

    def test_equality(self, chat_member):
        a = chat_member
        b = ChatMember(self.user, self.status)
        c = ChatMember(self.user, "unknown")
        d = ChatMember(User(2, "test_bot", is_bot=True), self.status)
        e = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture
def chat_member_administrator():
    return ChatMemberAdministrator(
        TestChatMemberAdministratorWithoutRequest.user,
        TestChatMemberAdministratorWithoutRequest.can_be_edited,
        TestChatMemberAdministratorWithoutRequest.can_change_info,
        TestChatMemberAdministratorWithoutRequest.can_delete_messages,
        TestChatMemberAdministratorWithoutRequest.can_delete_stories,
        TestChatMemberAdministratorWithoutRequest.can_edit_messages,
        TestChatMemberAdministratorWithoutRequest.can_edit_stories,
        TestChatMemberAdministratorWithoutRequest.can_invite_users,
        TestChatMemberAdministratorWithoutRequest.can_manage_chat,
        TestChatMemberAdministratorWithoutRequest.can_manage_topics,
        TestChatMemberAdministratorWithoutRequest.can_manage_video_chats,
        TestChatMemberAdministratorWithoutRequest.can_pin_messages,
        TestChatMemberAdministratorWithoutRequest.can_post_messages,
        TestChatMemberAdministratorWithoutRequest.can_post_stories,
        TestChatMemberAdministratorWithoutRequest.can_promote_members,
        TestChatMemberAdministratorWithoutRequest.can_restrict_members,
        TestChatMemberAdministratorWithoutRequest.custom_title,
        TestChatMemberAdministratorWithoutRequest.is_anonymous,
        TestChatMemberAdministratorWithoutRequest.can_manage_direct_messages,
        TestChatMemberAdministratorWithoutRequest.can_manage_tags,
    )


class TestChatMemberAdministratorWithoutRequest(ChatMemberTestBase):
    status = ChatMemberStatus.ADMINISTRATOR

    def test_slot_behaviour(self, chat_member_administrator):
        inst = chat_member_administrator
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        data = {
            "user": self.user.to_dict(),
            "can_be_edited": self.can_be_edited,
            "can_change_info": self.can_change_info,
            "can_delete_messages": self.can_delete_messages,
            "can_delete_stories": self.can_delete_stories,
            "can_edit_messages": self.can_edit_messages,
            "can_edit_stories": self.can_edit_stories,
            "can_invite_users": self.can_invite_users,
            "can_manage_chat": self.can_manage_chat,
            "can_manage_topics": self.can_manage_topics,
            "can_manage_video_chats": self.can_manage_video_chats,
            "can_pin_messages": self.can_pin_messages,
            "can_post_messages": self.can_post_messages,
            "can_post_stories": self.can_post_stories,
            "can_promote_members": self.can_promote_members,
            "can_restrict_members": self.can_restrict_members,
            "custom_title": self.custom_title,
            "is_anonymous": self.is_anonymous,
            "can_manage_direct_messages": self.can_manage_direct_messages,
            "can_manage_tags": self.can_manage_tags,
        }
        chat_member = ChatMemberAdministrator.de_json(data, offline_bot)

        assert type(chat_member) is ChatMemberAdministrator
        assert chat_member.api_kwargs == {}

        assert chat_member.user == self.user
        assert chat_member.can_be_edited == self.can_be_edited
        assert chat_member.can_change_info == self.can_change_info
        assert chat_member.can_delete_messages == self.can_delete_messages
        assert chat_member.can_delete_stories == self.can_delete_stories
        assert chat_member.can_edit_messages == self.can_edit_messages
        assert chat_member.can_edit_stories == self.can_edit_stories
        assert chat_member.can_invite_users == self.can_invite_users
        assert chat_member.can_manage_chat == self.can_manage_chat
        assert chat_member.can_manage_topics == self.can_manage_topics
        assert chat_member.can_manage_video_chats == self.can_manage_video_chats
        assert chat_member.can_pin_messages == self.can_pin_messages
        assert chat_member.can_post_messages == self.can_post_messages
        assert chat_member.can_post_stories == self.can_post_stories
        assert chat_member.can_promote_members == self.can_promote_members
        assert chat_member.can_restrict_members == self.can_restrict_members
        assert chat_member.custom_title == self.custom_title
        assert chat_member.is_anonymous == self.is_anonymous
        assert chat_member.can_manage_direct_messages == self.can_manage_direct_messages
        assert chat_member.can_manage_tags == self.can_manage_tags

    def test_to_dict(self, chat_member_administrator):
        assert chat_member_administrator.to_dict() == {
            "status": chat_member_administrator.status,
            "user": chat_member_administrator.user.to_dict(),
            "can_be_edited": chat_member_administrator.can_be_edited,
            "can_change_info": chat_member_administrator.can_change_info,
            "can_delete_messages": chat_member_administrator.can_delete_messages,
            "can_delete_stories": chat_member_administrator.can_delete_stories,
            "can_edit_messages": chat_member_administrator.can_edit_messages,
            "can_edit_stories": chat_member_administrator.can_edit_stories,
            "can_invite_users": chat_member_administrator.can_invite_users,
            "can_manage_chat": chat_member_administrator.can_manage_chat,
            "can_manage_topics": chat_member_administrator.can_manage_topics,
            "can_manage_video_chats": chat_member_administrator.can_manage_video_chats,
            "can_pin_messages": chat_member_administrator.can_pin_messages,
            "can_post_messages": chat_member_administrator.can_post_messages,
            "can_post_stories": chat_member_administrator.can_post_stories,
            "can_promote_members": chat_member_administrator.can_promote_members,
            "can_restrict_members": chat_member_administrator.can_restrict_members,
            "custom_title": chat_member_administrator.custom_title,
            "is_anonymous": chat_member_administrator.is_anonymous,
            "can_manage_direct_messages": chat_member_administrator.can_manage_direct_messages,
            "can_manage_tags": chat_member_administrator.can_manage_tags,
        }

    def test_equality(self, chat_member_administrator):
        a = chat_member_administrator
        b = ChatMemberAdministrator(
            User(1, "test_user", is_bot=False),
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
            True,
        )
        c = ChatMemberAdministrator(
            User(1, "test_user", is_bot=False),
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
        )
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def chat_member_banned():
    return ChatMemberBanned(
        TestChatMemberBannedWithoutRequest.user,
        TestChatMemberBannedWithoutRequest.until_date,
    )


class TestChatMemberBannedWithoutRequest(ChatMemberTestBase):
    status = ChatMemberStatus.BANNED

    def test_slot_behaviour(self, chat_member_banned):
        inst = chat_member_banned
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        data = {
            "user": self.user.to_dict(),
            "until_date": to_timestamp(self.until_date),
        }
        chat_member = ChatMemberBanned.de_json(data, offline_bot)

        assert type(chat_member) is ChatMemberBanned
        assert chat_member.api_kwargs == {}

        assert chat_member.user == self.user
        assert chat_member.until_date == self.until_date

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "user": self.user.to_dict(),
            "until_date": to_timestamp(self.until_date),
        }

        cmb_raw = ChatMemberBanned.de_json(json_dict, raw_bot)
        cmb_bot = ChatMemberBanned.de_json(json_dict, offline_bot)
        cmb_bot_tz = ChatMemberBanned.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        cmb_bot_tz_offset = cmb_bot_tz.until_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            cmb_bot_tz.until_date.replace(tzinfo=None)
        )

        assert cmb_raw.until_date.tzinfo == UTC
        assert cmb_bot.until_date.tzinfo == UTC
        assert cmb_bot_tz_offset == tz_bot_offset

    def test_to_dict(self, chat_member_banned):
        assert chat_member_banned.to_dict() == {
            "status": chat_member_banned.status,
            "user": chat_member_banned.user.to_dict(),
            "until_date": to_timestamp(chat_member_banned.until_date),
        }

    def test_equality(self, chat_member_banned):
        a = chat_member_banned
        b = ChatMemberBanned(
            User(1, "test_user", is_bot=False), dtm.datetime.now(UTC).replace(microsecond=0)
        )
        c = ChatMemberBanned(
            User(2, "test_bot", is_bot=True), dtm.datetime.now(UTC).replace(microsecond=0)
        )
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def chat_member_left():
    return ChatMemberLeft(TestChatMemberLeftWithoutRequest.user)


class TestChatMemberLeftWithoutRequest(ChatMemberTestBase):
    status = ChatMemberStatus.LEFT

    def test_slot_behaviour(self, chat_member_left):
        inst = chat_member_left
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        data = {"user": self.user.to_dict()}
        chat_member = ChatMemberLeft.de_json(data, offline_bot)

        assert type(chat_member) is ChatMemberLeft
        assert chat_member.api_kwargs == {}

        assert chat_member.user == self.user

    def test_to_dict(self, chat_member_left):
        assert chat_member_left.to_dict() == {
            "status": chat_member_left.status,
            "user": chat_member_left.user.to_dict(),
        }

    def test_equality(self, chat_member_left):
        a = chat_member_left
        b = ChatMemberLeft(User(1, "test_user", is_bot=False))
        c = ChatMemberLeft(User(2, "test_bot", is_bot=True))
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def chat_member_member():
    return ChatMemberMember(TestChatMemberMemberWithoutRequest.user)


class TestChatMemberMemberWithoutRequest(ChatMemberTestBase):
    status = ChatMemberStatus.MEMBER

    def test_slot_behaviour(self, chat_member_member):
        inst = chat_member_member
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        data = {"user": self.user.to_dict(), "until_date": to_timestamp(self.until_date)}
        chat_member = ChatMemberMember.de_json(data, offline_bot)

        assert type(chat_member) is ChatMemberMember
        assert chat_member.api_kwargs == {}

        assert chat_member.user == self.user
        assert chat_member.until_date == self.until_date

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "user": self.user.to_dict(),
            "until_date": to_timestamp(self.until_date),
        }

        cmm_raw = ChatMemberMember.de_json(json_dict, raw_bot)
        cmm_bot = ChatMemberMember.de_json(json_dict, offline_bot)
        cmm_bot_tz = ChatMemberMember.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        cmm_bot_tz_offset = cmm_bot_tz.until_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            cmm_bot_tz.until_date.replace(tzinfo=None)
        )

        assert cmm_raw.until_date.tzinfo == UTC
        assert cmm_bot.until_date.tzinfo == UTC
        assert cmm_bot_tz_offset == tz_bot_offset

    def test_to_dict(self, chat_member_member):
        assert chat_member_member.to_dict() == {
            "status": chat_member_member.status,
            "user": chat_member_member.user.to_dict(),
        }

    def test_equality(self, chat_member_member):
        a = chat_member_member
        b = ChatMemberMember(User(1, "test_user", is_bot=False))
        c = ChatMemberMember(User(2, "test_bot", is_bot=True))
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def chat_member_owner():
    return ChatMemberOwner(
        TestChatMemberOwnerWithoutRequest.user,
        TestChatMemberOwnerWithoutRequest.is_anonymous,
        TestChatMemberOwnerWithoutRequest.custom_title,
    )


class TestChatMemberOwnerWithoutRequest(ChatMemberTestBase):
    status = ChatMemberStatus.OWNER

    def test_slot_behaviour(self, chat_member_owner):
        inst = chat_member_owner
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        data = {
            "user": self.user.to_dict(),
            "is_anonymous": self.is_anonymous,
            "custom_title": self.custom_title,
        }
        chat_member = ChatMemberOwner.de_json(data, offline_bot)

        assert type(chat_member) is ChatMemberOwner
        assert chat_member.api_kwargs == {}

        assert chat_member.user == self.user
        assert chat_member.is_anonymous == self.is_anonymous
        assert chat_member.custom_title == self.custom_title

    def test_to_dict(self, chat_member_owner):
        assert chat_member_owner.to_dict() == {
            "status": chat_member_owner.status,
            "user": chat_member_owner.user.to_dict(),
            "is_anonymous": chat_member_owner.is_anonymous,
            "custom_title": chat_member_owner.custom_title,
        }

    def test_equality(self, chat_member_owner):
        a = chat_member_owner
        b = ChatMemberOwner(User(1, "test_user", is_bot=False), True, "test_title")
        c = ChatMemberOwner(User(1, "test_user", is_bot=False), False, "test_title")
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture
def chat_member_restricted():
    return ChatMemberRestricted(
        user=TestChatMemberRestrictedWithoutRequest.user,
        can_add_web_page_previews=TestChatMemberRestrictedWithoutRequest.can_add_web_page_previews,
        can_change_info=TestChatMemberRestrictedWithoutRequest.can_change_info,
        can_invite_users=TestChatMemberRestrictedWithoutRequest.can_invite_users,
        can_manage_topics=TestChatMemberRestrictedWithoutRequest.can_manage_topics,
        can_pin_messages=TestChatMemberRestrictedWithoutRequest.can_pin_messages,
        can_send_audios=TestChatMemberRestrictedWithoutRequest.can_send_audios,
        can_send_documents=TestChatMemberRestrictedWithoutRequest.can_send_documents,
        can_send_messages=TestChatMemberRestrictedWithoutRequest.can_send_messages,
        can_send_other_messages=TestChatMemberRestrictedWithoutRequest.can_send_other_messages,
        can_send_photos=TestChatMemberRestrictedWithoutRequest.can_send_photos,
        can_send_polls=TestChatMemberRestrictedWithoutRequest.can_send_polls,
        can_send_video_notes=TestChatMemberRestrictedWithoutRequest.can_send_video_notes,
        can_send_videos=TestChatMemberRestrictedWithoutRequest.can_send_videos,
        can_send_voice_notes=TestChatMemberRestrictedWithoutRequest.can_send_voice_notes,
        is_member=TestChatMemberRestrictedWithoutRequest.is_member,
        until_date=TestChatMemberRestrictedWithoutRequest.until_date,
        can_edit_tag=TestChatMemberRestrictedWithoutRequest.can_edit_tag,
        tag=TestChatMemberRestrictedWithoutRequest.tag,
    )


class TestChatMemberRestrictedWithoutRequest(ChatMemberTestBase):
    status = ChatMemberStatus.RESTRICTED

    def test_slot_behaviour(self, chat_member_restricted):
        inst = chat_member_restricted
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        data = {
            "user": self.user.to_dict(),
            "can_add_web_page_previews": self.can_add_web_page_previews,
            "can_change_info": self.can_change_info,
            "can_invite_users": self.can_invite_users,
            "can_manage_topics": self.can_manage_topics,
            "can_pin_messages": self.can_pin_messages,
            "can_send_audios": self.can_send_audios,
            "can_send_documents": self.can_send_documents,
            "can_send_messages": self.can_send_messages,
            "can_send_other_messages": self.can_send_other_messages,
            "can_send_photos": self.can_send_photos,
            "can_send_polls": self.can_send_polls,
            "can_send_video_notes": self.can_send_video_notes,
            "can_send_videos": self.can_send_videos,
            "can_send_voice_notes": self.can_send_voice_notes,
            "is_member": self.is_member,
            "until_date": to_timestamp(self.until_date),
            "can_edit_tag": self.can_edit_tag,
            "tag": self.tag,
            # legacy argument
            "can_send_media_messages": False,
        }
        chat_member = ChatMemberRestricted.de_json(data, offline_bot)

        assert type(chat_member) is ChatMemberRestricted
        assert chat_member.api_kwargs == {"can_send_media_messages": False}

        assert chat_member.user == self.user
        assert chat_member.can_add_web_page_previews == self.can_add_web_page_previews
        assert chat_member.can_change_info == self.can_change_info
        assert chat_member.can_invite_users == self.can_invite_users
        assert chat_member.can_manage_topics == self.can_manage_topics
        assert chat_member.can_pin_messages == self.can_pin_messages
        assert chat_member.can_send_audios == self.can_send_audios
        assert chat_member.can_send_documents == self.can_send_documents
        assert chat_member.can_send_messages == self.can_send_messages
        assert chat_member.can_send_other_messages == self.can_send_other_messages
        assert chat_member.can_send_photos == self.can_send_photos
        assert chat_member.can_send_polls == self.can_send_polls
        assert chat_member.can_send_video_notes == self.can_send_video_notes
        assert chat_member.can_send_videos == self.can_send_videos
        assert chat_member.can_send_voice_notes == self.can_send_voice_notes
        assert chat_member.is_member == self.is_member
        assert chat_member.until_date == self.until_date
        assert chat_member.can_edit_tag == self.can_edit_tag
        assert chat_member.tag == self.tag

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot, chat_member_restricted):
        json_dict = chat_member_restricted.to_dict()

        cmr_raw = ChatMemberRestricted.de_json(json_dict, raw_bot)
        cmr_bot = ChatMemberRestricted.de_json(json_dict, offline_bot)
        cmr_bot_tz = ChatMemberRestricted.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        cmr_bot_tz_offset = cmr_bot_tz.until_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            cmr_bot_tz.until_date.replace(tzinfo=None)
        )

        assert cmr_raw.until_date.tzinfo == UTC
        assert cmr_bot.until_date.tzinfo == UTC
        assert cmr_bot_tz_offset == tz_bot_offset

    def test_to_dict(self, chat_member_restricted):
        assert chat_member_restricted.to_dict() == {
            "status": chat_member_restricted.status,
            "user": chat_member_restricted.user.to_dict(),
            "can_add_web_page_previews": chat_member_restricted.can_add_web_page_previews,
            "can_change_info": chat_member_restricted.can_change_info,
            "can_invite_users": chat_member_restricted.can_invite_users,
            "can_manage_topics": chat_member_restricted.can_manage_topics,
            "can_pin_messages": chat_member_restricted.can_pin_messages,
            "can_send_audios": chat_member_restricted.can_send_audios,
            "can_send_documents": chat_member_restricted.can_send_documents,
            "can_send_messages": chat_member_restricted.can_send_messages,
            "can_send_other_messages": chat_member_restricted.can_send_other_messages,
            "can_send_photos": chat_member_restricted.can_send_photos,
            "can_send_polls": chat_member_restricted.can_send_polls,
            "can_send_video_notes": chat_member_restricted.can_send_video_notes,
            "can_send_videos": chat_member_restricted.can_send_videos,
            "can_send_voice_notes": chat_member_restricted.can_send_voice_notes,
            "is_member": chat_member_restricted.is_member,
            "until_date": to_timestamp(chat_member_restricted.until_date),
            "can_edit_tag": chat_member_restricted.can_edit_tag,
            "tag": chat_member_restricted.tag,
        }

    def test_equality(self, chat_member_restricted):
        a = chat_member_restricted
        b = deepcopy(chat_member_restricted)
        c = ChatMemberRestricted(
            User(1, "test_user", is_bot=False),
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            self.until_date,
            False,
            False,
            False,
            False,
            False,
            False,
            False,
            "tag",
        )
        d = Dice(5, "test")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm

import pytest

from telegram import (
    Audio,
    Bot,
    CallbackQuery,
    Chat,
    InaccessibleMessage,
    InputChecklist,
    InputChecklistTask,
    Message,
    User,
)
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(params=["message", "inline", "inaccessible_message"])
def callback_query(bot, request):
    cbq = CallbackQuery(
        CallbackQueryTestBase.id_,
        CallbackQueryTestBase.from_user,
        CallbackQueryTestBase.chat_instance,
        data=CallbackQueryTestBase.data,
        game_short_name=CallbackQueryTestBase.game_short_name,
    )
    cbq.set_bot(bot)
    cbq._unfreeze()
    if request.param == "message":
        cbq.message = CallbackQueryTestBase.message
        cbq.message.set_bot(bot)
    elif request.param == "inline":
        cbq.inline_message_id = CallbackQueryTestBase.inline_message_id
    elif request.param == "inaccessible_message":
        cbq.message = InaccessibleMessage(
            chat=CallbackQueryTestBase.message.chat,
            message_id=CallbackQueryTestBase.message.message_id,
        )
    return cbq


class CallbackQueryTestBase:
    id_ = "id"
    from_user = User(1, "test_user", False)
    chat_instance = "chat_instance"
    message = Message(
        3, dtm.datetime.utcnow(), Chat(4, "private"), from_user=User(5, "bot", False)
    )
    data = "data"
    inline_message_id = "inline_message_id"
    game_short_name = "the_game"


class TestCallbackQueryWithoutRequest(CallbackQueryTestBase):
    @staticmethod
    def skip_params(callback_query: CallbackQuery):
        if callback_query.inline_message_id:
            return {"message_id", "chat_id", "business_connection_id"}
        return {"inline_message_id", "business_connection_id"}

    @staticmethod
    def shortcut_kwargs(callback_query: CallbackQuery):
        if not callback_query.inline_message_id:
            return {"message_id", "chat_id"}
        return {"inline_message_id"}

    @staticmethod
    def check_passed_ids(callback_query: CallbackQuery, kwargs):
        if callback_query.inline_message_id:
            id_ = kwargs["inline_message_id"] == callback_query.inline_message_id
            chat_id = kwargs["chat_id"] is None
            message_id = kwargs["message_id"] is None
        else:
            id_ = kwargs["inline_message_id"] is None
            chat_id = kwargs["chat_id"] == callback_query.message.chat_id
            message_id = kwargs["message_id"] == callback_query.message.message_id
        return id_ and chat_id and message_id

    def test_slot_behaviour(self, callback_query):
        for attr in callback_query.__slots__:
            assert getattr(callback_query, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(callback_query)) == len(set(mro_slots(callback_query))), "same slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id_,
            "from": self.from_user.to_dict(),
            "chat_instance": self.chat_instance,
            "message": self.message.to_dict(),
            "data": self.data,
            "inline_message_id": self.inline_message_id,
            "game_short_name": self.game_short_name,
        }
        callback_query = CallbackQuery.de_json(json_dict, offline_bot)
        assert callback_query.api_kwargs == {}

        assert callback_query.id == self.id_
        assert callback_query.from_user == self.from_user
        assert callback_query.chat_instance == self.chat_instance
        assert callback_query.message == self.message
        assert callback_query.data == self.data
        assert callback_query.inline_message_id == self.inline_message_id
        assert callback_query.game_short_name == self.game_short_name

    def test_to_dict(self, callback_query):
        callback_query_dict = callback_query.to_dict()

        assert isinstance(callback_query_dict, dict)
        assert callback_query_dict["id"] == callback_query.id
        assert callback_query_dict["from"] == callback_query.from_user.to_dict()
        assert callback_query_dict["chat_instance"] == callback_query.chat_instance
        if callback_query.message is not None:
            assert callback_query_dict["message"] == callback_query.message.to_dict()
        elif callback_query.inline_message_id:
            assert callback_query_dict["inline_message_id"] == callback_query.inline_message_id
        assert callback_query_dict["data"] == callback_query.data
        assert callback_query_dict["game_short_name"] == callback_query.game_short_name

    def test_equality(self):
        a = CallbackQuery(self.id_, self.from_user, "chat")
        b = CallbackQuery(self.id_, self.from_user, "chat")
        c = CallbackQuery(self.id_, None, "")
        d = CallbackQuery("", None, "chat")
        e = Audio(self.id_, "unique_id", 1)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_answer(self, monkeypatch, callback_query):
        async def make_assertion(*_, **kwargs):
            return kwargs["callback_query_id"] == callback_query.id

        assert check_shortcut_signature(
            CallbackQuery.answer, Bot.answer_callback_query, ["callback_query_id"], []
        )
        assert await check_shortcut_call(
            callback_query.answer, callback_query.get_bot(), "answer_callback_query"
        )
        assert await check_defaults_handling(callback_query.answer, callback_query.get_bot())

        monkeypatch.setattr(callback_query.get_bot(), "answer_callback_query", make_assertion)
        assert await callback_query.answer()

    async def test_edit_message_text(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.edit_message_text("test")
            return

        async def make_assertion(*_, **kwargs):
            text = kwargs["text"] == "test"
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and text

        assert check_shortcut_signature(
            CallbackQuery.edit_message_text,
            Bot.edit_message_text,
            ["inline_message_id", "message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.edit_message_text,
            callback_query.get_bot(),
            "edit_message_text",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.edit_message_text, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "edit_message_text", make_assertion)
        assert await callback_query.edit_message_text(text="test")
        assert await callback_query.edit_message_text("test")

    async def test_edit_message_caption(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.edit_message_caption("test")
            return

        async def make_assertion(*_, **kwargs):
            caption = kwargs["caption"] == "new caption"
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and caption

        assert check_shortcut_signature(
            CallbackQuery.edit_message_caption,
            Bot.edit_message_caption,
            ["inline_message_id", "message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.edit_message_caption,
            callback_query.get_bot(),
            "edit_message_caption",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.edit_message_caption, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "edit_message_caption", make_assertion)
        assert await callback_query.edit_message_caption(caption="new caption")
        assert await callback_query.edit_message_caption("new caption")

    async def test_edit_message_checklist(self, monkeypatch, callback_query):
        checklist = InputChecklist(title="My Checklist", tasks=[InputChecklistTask(1, "Task 1")])

        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.edit_message_checklist(checklist)
            return

        if callback_query.inline_message_id:
            pytest.skip("Can't edit inline messages")

        async def make_assertion(*_, **kwargs):
            chat_id = kwargs["chat_id"] == callback_query.message.chat_id
            message_id = kwargs["message_id"] == callback_query.message.message_id
            caption = kwargs["checklist"] == checklist
            return chat_id and message_id and caption

        assert check_shortcut_signature(
            CallbackQuery.edit_message_checklist,
            Bot.edit_message_checklist,
            ["chat_id", "message_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.edit_message_checklist,
            callback_query.get_bot(),
            "edit_message_checklist",
            shortcut_kwargs=["chat_id", "message_id", "business_connection_id"],
        )
        assert await check_defaults_handling(
            callback_query.edit_message_checklist, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "edit_message_checklist", make_assertion)
        assert await callback_query.edit_message_checklist(checklist=checklist)
        assert await callback_query.edit_message_checklist(checklist)

    async def test_edit_message_reply_markup(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.edit_message_reply_markup("test")
            return

        async def make_assertion(*_, **kwargs):
            reply_markup = kwargs["reply_markup"] == [["1", "2"]]
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and reply_markup

        assert check_shortcut_signature(
            CallbackQuery.edit_message_reply_markup,
            Bot.edit_message_reply_markup,
            ["inline_message_id", "message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.edit_message_reply_markup,
            callback_query.get_bot(),
            "edit_message_reply_markup",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.edit_message_reply_markup, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "edit_message_reply_markup", make_assertion)
        assert await callback_query.edit_message_reply_markup(reply_markup=[["1", "2"]])
        assert await callback_query.edit_message_reply_markup([["1", "2"]])

    async def test_edit_message_media(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.edit_message_media("test")
            return

        async def make_assertion(*_, **kwargs):
            message_media = kwargs.get("media") == [["1", "2"]]
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and message_media

        assert check_shortcut_signature(
            CallbackQuery.edit_message_media,
            Bot.edit_message_media,
            ["inline_message_id", "message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.edit_message_media,
            callback_query.get_bot(),
            "edit_message_media",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.edit_message_media, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "edit_message_media", make_assertion)
        assert await callback_query.edit_message_media(media=[["1", "2"]])
        assert await callback_query.edit_message_media([["1", "2"]])

    async def test_edit_message_live_location(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.edit_message_live_location("test")
            return

        async def make_assertion(*_, **kwargs):
            latitude = kwargs.get("latitude") == 1
            longitude = kwargs.get("longitude") == 2
            live = kwargs.get("live_period") == 900
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and latitude and longitude and live

        assert check_shortcut_signature(
            CallbackQuery.edit_message_live_location,
            Bot.edit_message_live_location,
            ["inline_message_id", "message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.edit_message_live_location,
            callback_query.get_bot(),
            "edit_message_live_location",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.edit_message_live_location, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "edit_message_live_location", make_assertion)
        assert await callback_query.edit_message_live_location(
            latitude=1, longitude=2, live_period=900
        )
        assert await callback_query.edit_message_live_location(1, 2, live_period=900)

    async def test_stop_message_live_location(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.stop_message_live_location("test")
            return

        async def make_assertion(*_, **kwargs):
            return self.check_passed_ids(callback_query, kwargs)

        assert check_shortcut_signature(
            CallbackQuery.stop_message_live_location,
            Bot.stop_message_live_location,
            ["inline_message_id", "message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.stop_message_live_location,
            callback_query.get_bot(),
            "stop_message_live_location",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.stop_message_live_location, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "stop_message_live_location", make_assertion)
        assert await callback_query.stop_message_live_location()

    async def test_set_game_score(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.set_game_score(user_id=1, score=2)
            return

        async def make_assertion(*_, **kwargs):
            user_id = kwargs.get("user_id") == 1
            score = kwargs.get("score") == 2
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and user_id and score

        assert check_shortcut_signature(
            CallbackQuery.set_game_score,
            Bot.set_game_score,
            ["inline_message_id", "message_id", "chat_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.set_game_score,
            callback_query.get_bot(),
            "set_game_score",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.set_game_score, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "set_game_score", make_assertion)
        assert await callback_query.set_game_score(user_id=1, score=2)
        assert await callback_query.set_game_score(1, 2)

    async def test_get_game_high_scores(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.get_game_high_scores("test")
            return

        async def make_assertion(*_, **kwargs):
            user_id = kwargs.get("user_id") == 1
            ids = self.check_passed_ids(callback_query, kwargs)
            return ids and user_id

        assert check_shortcut_signature(
            CallbackQuery.get_game_high_scores,
            Bot.get_game_high_scores,
            ["inline_message_id", "message_id", "chat_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.get_game_high_scores,
            callback_query.get_bot(),
            "get_game_high_scores",
            skip_params=self.skip_params(callback_query),
            shortcut_kwargs=self.shortcut_kwargs(callback_query),
        )
        assert await check_defaults_handling(
            callback_query.get_game_high_scores, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "get_game_high_scores", make_assertion)
        assert await callback_query.get_game_high_scores(user_id=1)
        assert await callback_query.get_game_high_scores(1)

    async def test_delete_message(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.delete_message()
            return
        if callback_query.inline_message_id:
            pytest.skip("Can't delete inline messages")

        async def make_assertion(*args, **kwargs):
            id_ = kwargs["chat_id"] == callback_query.message.chat_id
            message = kwargs["message_id"] == callback_query.message.message_id
            return id_ and message

        assert check_shortcut_signature(
            CallbackQuery.delete_message,
            Bot.delete_message,
            ["message_id", "chat_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.delete_message, callback_query.get_bot(), "delete_message"
        )
        assert await check_defaults_handling(
            callback_query.delete_message, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "delete_message", make_assertion)
        assert await callback_query.delete_message()

    async def test_pin_message(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.pin_message()
            return
        if callback_query.inline_message_id:
            pytest.skip("Can't pin inline messages")

        async def make_assertion(*args, **kwargs):
            return kwargs["chat_id"] == callback_query.message.chat_id

        assert check_shortcut_signature(
            CallbackQuery.pin_message,
            Bot.pin_chat_message,
            ["message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.pin_message,
            callback_query.get_bot(),
            "pin_chat_message",
            ["business_connection_id"],
        )
        assert await check_defaults_handling(callback_query.pin_message, callback_query.get_bot())

        monkeypatch.setattr(callback_query.get_bot(), "pin_chat_message", make_assertion)
        assert await callback_query.pin_message()

    async def test_unpin_message(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.unpin_message()
            return
        if callback_query.inline_message_id:
            pytest.skip("Can't unpin inline messages")

        async def make_assertion(*args, **kwargs):
            return kwargs["chat_id"] == callback_query.message.chat_id

        assert check_shortcut_signature(
            CallbackQuery.unpin_message,
            Bot.unpin_chat_message,
            ["message_id", "chat_id", "business_connection_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.unpin_message,
            callback_query.get_bot(),
            "unpin_chat_message",
            shortcut_kwargs=["message_id"],
            skip_params=["business_connection_id"],
        )
        assert await check_defaults_handling(
            callback_query.unpin_message, callback_query.get_bot()
        )

        monkeypatch.setattr(callback_query.get_bot(), "unpin_chat_message", make_assertion)
        assert await callback_query.unpin_message()

    async def test_copy_message(self, monkeypatch, callback_query):
        if isinstance(callback_query.message, InaccessibleMessage):
            with pytest.raises(TypeError, match="inaccessible message"):
                await callback_query.copy_message(1)
            return
        if callback_query.inline_message_id:
            pytest.skip("Can't copy inline messages")

        async def make_assertion(*args, **kwargs):
            id_ = kwargs["from_chat_id"] == callback_query.message.chat_id
            chat_id = kwargs["chat_id"] == 1
            message = kwargs["message_id"] == callback_query.message.message_id
            return id_ and message and chat_id

        assert check_shortcut_signature(
            CallbackQuery.copy_message,
            Bot.copy_message,
            ["message_id", "from_chat_id", "direct_messages_topic_id"],
            [],
        )
        assert await check_shortcut_call(
            callback_query.copy_message,
            callback_query.get_bot(),
            "copy_message",
            shortcut_kwargs=["direct_messages_topic_id"],
        )
        assert await check_defaults_handling(callback_query.copy_message, callback_query.get_bot())

        monkeypatch.setattr(callback_query.get_bot(), "copy_message", make_assertion)
        assert await callback_query.copy_message(1)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains two objects to request chats/users."""

from typing import TYPE_CHECKING

from telegram._chatadministratorrights import ChatAdministratorRights
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class KeyboardButtonRequestUsers(TelegramObject):
    """This object defines the criteria used to request a suitable user. The identifier of the
    selected user will be shared with the bot when the corresponding button is pressed. `More
    about requesting users » <https://core.telegram.org/bots/features#chat-and-user-selection>`_.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`request_id` is equal.

    .. versionadded:: 20.8
        This class was previously named ``KeyboardButtonRequestUser``.

    Args:
        request_id (:obj:`int`): Signed 32-bit identifier of the request, which will be received
            back in the :class:`telegram.UsersShared` object. Must be unique within the message.
        user_is_bot (:obj:`bool`, optional): Pass :obj:`True` to request a bot, pass :obj:`False`
            to request a regular user. If not specified, no additional restrictions are applied.
        user_is_premium (:obj:`bool`, optional): Pass :obj:`True` to request a premium user, pass
            :obj:`False` to request a non-premium user. If not specified, no additional
            restrictions are applied.
        max_quantity (:obj:`int`, optional): The maximum number of users to be selected;
            :tg-const:`telegram.constants.KeyboardButtonRequestUsersLimit.MIN_QUANTITY` -
            :tg-const:`telegram.constants.KeyboardButtonRequestUsersLimit.MAX_QUANTITY`.
            Defaults to :tg-const:`telegram.constants.KeyboardButtonRequestUsersLimit.MIN_QUANTITY`
            .

            .. versionadded:: 20.8
        request_name (:obj:`bool`, optional): Pass :obj:`True` to request the users' first and last
            name.

            .. versionadded:: 21.1
        request_username (:obj:`bool`, optional): Pass :obj:`True` to request the users' username.

            .. versionadded:: 21.1
        request_photo (:obj:`bool`, optional): Pass :obj:`True` to request the users' photo.

            .. versionadded:: 21.1

    Attributes:
        request_id (:obj:`int`): Identifier of the request.
        user_is_bot (:obj:`bool`): Optional. Pass :obj:`True` to request a bot, pass :obj:`False`
            to request a regular user. If not specified, no additional restrictions are applied.
        user_is_premium (:obj:`bool`): Optional. Pass :obj:`True` to request a premium user, pass
            :obj:`False` to request a non-premium user. If not specified, no additional
            restrictions are applied.
        max_quantity (:obj:`int`): Optional. The maximum number of users to be selected;
            :tg-const:`telegram.constants.KeyboardButtonRequestUsersLimit.MIN_QUANTITY` -
            :tg-const:`telegram.constants.KeyboardButtonRequestUsersLimit.MAX_QUANTITY`.
            Defaults to :tg-const:`telegram.constants.KeyboardButtonRequestUsersLimit.MIN_QUANTITY`
            .

            .. versionadded:: 20.8
        request_name (:obj:`bool`): Optional. Pass :obj:`True` to request the users' first and last
            name.

            .. versionadded:: 21.1
        request_username (:obj:`bool`): Optional. Pass :obj:`True` to request the users' username.

            .. versionadded:: 21.1
        request_photo (:obj:`bool`): Optional. Pass :obj:`True` to request the users' photo.

            .. versionadded:: 21.1

    """

    __slots__ = (
        "max_quantity",
        "request_id",
        "request_name",
        "request_photo",
        "request_username",
        "user_is_bot",
        "user_is_premium",
    )

    def __init__(
        self,
        request_id: int,
        user_is_bot: bool | None = None,
        user_is_premium: bool | None = None,
        max_quantity: int | None = None,
        request_name: bool | None = None,
        request_username: bool | None = None,
        request_photo: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.request_id: int = request_id

        # Optionals
        self.user_is_bot: bool | None = user_is_bot
        self.user_is_premium: bool | None = user_is_premium
        self.max_quantity: int | None = max_quantity
        self.request_name: bool | None = request_name
        self.request_username: bool | None = request_username
        self.request_photo: bool | None = request_photo

        self._id_attrs = (self.request_id,)

        self._freeze()


class KeyboardButtonRequestChat(TelegramObject):
    """This object defines the criteria used to request a suitable chat. The identifier of the
    selected user will be shared with the bot when the corresponding button is pressed. `More
    about requesting users » <https://core.telegram.org/bots/features#chat-and-user-selection>`_.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`request_id` is equal.

    .. versionadded:: 20.1

    Args:
        request_id (:obj:`int`): Signed 32-bit identifier of the request, which will be received
            back in the :class:`telegram.ChatShared` object. Must be unique within the message.
        chat_is_channel (:obj:`bool`): Pass :obj:`True` to request a channel chat, pass
            :obj:`False` to request a group or a supergroup chat.
        chat_is_forum (:obj:`bool`, optional): Pass :obj:`True` to request a forum supergroup, pass
            :obj:`False` to request a non-forum chat. If not specified, no additional
            restrictions are applied.
        chat_has_username (:obj:`bool`, optional): Pass :obj:`True` to request a supergroup or a
            channel with a username, pass :obj:`False` to request a chat without a username. If
            not specified, no additional restrictions are applied.
        chat_is_created (:obj:`bool`, optional): Pass :obj:`True` to request a chat owned by the
            user. Otherwise, no additional restrictions are applied.
        user_administrator_rights (:class:`ChatAdministratorRights`, optional): Specifies the
            required administrator rights of the user in the chat. If not specified, no additional
            restrictions are applied.
        bot_administrator_rights (:class:`ChatAdministratorRights`, optional): Specifies the
            required administrator rights of the bot in the chat. The rights must be a subset of
            :paramref:`user_administrator_rights`. If not specified, no additional restrictions are
            applied.
        bot_is_member (:obj:`bool`, optional): Pass :obj:`True` to request a chat with the bot
            as a member. Otherwise, no additional restrictions are applied.
        request_title (:obj:`bool`, optional): Pass :obj:`True` to request the chat's title.

            .. versionadded:: 21.1
        request_username (:obj:`bool`, optional): Pass :obj:`True` to request the chat's username.

            .. versionadded:: 21.1
        request_photo (:obj:`bool`, optional): Pass :obj:`True` to request the chat's photo.

            .. versionadded:: 21.1
    Attributes:
        request_id (:obj:`int`): Identifier of the request.
        chat_is_channel (:obj:`bool`): Pass :obj:`True` to request a channel chat, pass
            :obj:`False` to request a group or a supergroup chat.
        chat_is_forum (:obj:`bool`): Optional. Pass :obj:`True` to request a forum supergroup, pass
            :obj:`False` to request a non-forum chat. If not specified, no additional
            restrictions are applied.
        chat_has_username (:obj:`bool`): Optional. Pass :obj:`True` to request a supergroup or a
            channel with a username, pass :obj:`False` to request a chat without a username. If
            not specified, no additional restrictions are applied.
        chat_is_created (:obj:`bool`) Optional. Pass :obj:`True` to request a chat owned by the
            user. Otherwise, no additional restrictions are applied.
        user_administrator_rights (:class:`ChatAdministratorRights`) Optional. Specifies the
            required administrator rights of the user in the chat. If not specified, no additional
            restrictions are applied.
        bot_administrator_rights (:class:`ChatAdministratorRights`) Optional. Specifies the
            required administrator rights of the bot in the chat. The rights must be a subset of
            :attr:`user_administrator_rights`. If not specified, no additional restrictions are
            applied.
        bot_is_member (:obj:`bool`) Optional. Pass :obj:`True` to request a chat with the bot
            as a member. Otherwise, no additional restrictions are applied.
        request_title (:obj:`bool`): Optional. Pass :obj:`True` to request the chat's title.

            .. versionadded:: 21.1
        request_username (:obj:`bool`): Optional. Pass :obj:`True` to request the chat's username.

            .. versionadded:: 21.1
        request_photo (:obj:`bool`): Optional. Pass :obj:`True` to request the chat's photo.

            .. versionadded:: 21.1
    """

    __slots__ = (
        "bot_administrator_rights",
        "bot_is_member",
        "chat_has_username",
        "chat_is_channel",
        "chat_is_created",
        "chat_is_forum",
        "request_id",
        "request_photo",
        "request_title",
        "request_username",
        "user_administrator_rights",
    )

    def __init__(
        self,
        request_id: int,
        chat_is_channel: bool,
        chat_is_forum: bool | None = None,
        chat_has_username: bool | None = None,
        chat_is_created: bool | None = None,
        user_administrator_rights: ChatAdministratorRights | None = None,
        bot_administrator_rights: ChatAdministratorRights | None = None,
        bot_is_member: bool | None = None,
        request_title: bool | None = None,
        request_username: bool | None = None,
        request_photo: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # required
        self.request_id: int = request_id
        self.chat_is_channel: bool = chat_is_channel

        # optional
        self.chat_is_forum: bool | None = chat_is_forum
        self.chat_has_username: bool | None = chat_has_username
        self.chat_is_created: bool | None = chat_is_created
        self.user_administrator_rights: ChatAdministratorRights | None = user_administrator_rights
        self.bot_administrator_rights: ChatAdministratorRights | None = bot_administrator_rights
        self.bot_is_member: bool | None = bot_is_member
        self.request_title: bool | None = request_title
        self.request_username: bool | None = request_username
        self.request_photo: bool | None = request_photo

        self._id_attrs = (self.request_id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "KeyboardButtonRequestChat":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["user_administrator_rights"] = de_json_optional(
            data.get("user_administrator_rights"), ChatAdministratorRights, bot
        )
        data["bot_administrator_rights"] = de_json_optional(
            data.get("bot_administrator_rights"), ChatAdministratorRights, bot
        )

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the MessageHandler class."""

from typing import TYPE_CHECKING, Any, TypeVar

from telegram import Update
from telegram._utils.defaultvalue import DEFAULT_TRUE
from telegram._utils.types import DVType
from telegram.ext import filters as filters_module
from telegram.ext._handlers.basehandler import BaseHandler
from telegram.ext._utils.types import CCT, HandlerCallback

if TYPE_CHECKING:
    from telegram.ext import Application

RT = TypeVar("RT")


class MessageHandler(BaseHandler[Update, CCT, RT]):
    """Handler class to handle Telegram messages. They might contain text, media or status
    updates.

    Warning:
        When setting :paramref:`block` to :obj:`False`, you cannot rely on adding custom
        attributes to :class:`telegram.ext.CallbackContext`. See its docs for more info.

    Args:
        filters (:class:`telegram.ext.filters.BaseFilter`): A filter inheriting from
            :class:`telegram.ext.filters.BaseFilter`. Standard filters can be found in
            :mod:`telegram.ext.filters`. Filters can be combined using bitwise
            operators (& for and, | for or, ~ for not). Passing :obj:`None` is a shortcut
            to passing :class:`telegram.ext.filters.ALL`.

            .. seealso:: :wiki:`Advanced Filters <Extensions---Advanced-Filters>`
        callback (:term:`coroutine function`): The callback function for this handler. Will be
            called when :meth:`check_update` has determined that an update should be processed by
            this handler. Callback signature::

                async def callback(update: Update, context: CallbackContext)

            The return value of the callback is usually ignored except for the special case of
            :class:`telegram.ext.ConversationHandler`.
        block (:obj:`bool`, optional): Determines whether the return value of the callback should
            be awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`. Defaults to :obj:`True`.

            .. seealso:: :wiki:`Concurrency`

    Attributes:
        filters (:class:`telegram.ext.filters.BaseFilter`): Only allow updates with these Filters.
            See :mod:`telegram.ext.filters` for a full list of all available filters.
        callback (:term:`coroutine function`): The callback function for this handler.
        block (:obj:`bool`): Determines whether the return value of the callback should be
            awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`.

    """

    __slots__ = ("filters",)

    def __init__(
        self: "MessageHandler[CCT, RT]",
        filters: filters_module.BaseFilter | None,
        callback: HandlerCallback[Update, CCT, RT],
        block: DVType[bool] = DEFAULT_TRUE,
    ):
        super().__init__(callback, block=block)
        self.filters: filters_module.BaseFilter = (
            filters if filters is not None else filters_module.ALL
        )

    def check_update(self, update: object) -> bool | dict[str, list[Any]] | None:
        """Determines whether an update should be passed to this handler's :attr:`callback`.

        Args:
            update (:class:`telegram.Update` | :obj:`object`): Incoming update.

        Returns:
            :obj:`bool`

        """
        if isinstance(update, Update):
            return self.filters.check_update(update) or False
        return None

    def collect_additional_context(
        self,
        context: CCT,
        update: Update,  # noqa: ARG002
        application: "Application[Any, CCT, Any, Any, Any, Any]",  # noqa: ARG002
        check_result: bool | dict[str, object] | None,
    ) -> None:
        """Adds possible output of data filters to the :class:`CallbackContext`."""
        if isinstance(check_result, dict):
            context.update(check_result)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import time

import pytest

from telegram import (
    Chat,
    ChatBoost,
    ChatBoostRemoved,
    ChatBoostSourcePremium,
    ChatBoostUpdated,
    Update,
    User,
)
from telegram._utils.datetime import from_timestamp
from telegram.ext import CallbackContext, ChatBoostHandler
from tests.auxil.slots import mro_slots
from tests.test_update import all_types as really_all_types
from tests.test_update import params as all_params

# Remove "chat_boost" from params
params = [param for param in all_params for key in param if "chat_boost" not in key]
all_types = [param for param in really_all_types if "chat_boost" not in param]
ids = (*all_types, "callback_query_without_message")


@pytest.fixture(scope="class", params=params, ids=ids)
def false_update(request):
    return Update(update_id=2, **request.param)


def chat_boost():
    return ChatBoost(
        "1",
        from_timestamp(int(time.time())),
        from_timestamp(int(time.time())),
        ChatBoostSourcePremium(
            User(1, "first_name", False),
        ),
    )


@pytest.fixture(scope="module")
def removed_chat_boost():
    return ChatBoostRemoved(
        Chat(1, "group", username="chat"),
        "1",
        from_timestamp(int(time.time())),
        ChatBoostSourcePremium(
            User(1, "first_name", False),
        ),
    )


def removed_chat_boost_update():
    return Update(
        update_id=2,
        removed_chat_boost=ChatBoostRemoved(
            Chat(1, "group", username="chat"),
            "1",
            from_timestamp(int(time.time())),
            ChatBoostSourcePremium(
                User(1, "first_name", False),
            ),
        ),
    )


@pytest.fixture(scope="module")
def chat_boost_updated():
    return ChatBoostUpdated(Chat(1, "group", username="chat"), chat_boost())


def chat_boost_updated_update():
    return Update(
        update_id=2,
        chat_boost=ChatBoostUpdated(
            Chat(1, "group", username="chat"),
            chat_boost(),
        ),
    )


class TestChatBoostHandler:
    test_flag = False

    def test_slot_behaviour(self):
        action = ChatBoostHandler(self.cb_chat_boost_removed)
        for attr in action.__slots__:
            assert getattr(action, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(action)) == len(set(mro_slots(action))), "duplicate slot"

    @pytest.fixture(autouse=True)
    def _reset(self):
        self.test_flag = False

    async def cb_chat_boost_updated(self, update, context):
        self.test_flag = (
            isinstance(context, CallbackContext)
            and isinstance(update.chat_boost, ChatBoostUpdated)
            and not isinstance(update.removed_chat_boost, ChatBoostRemoved)
        )

    async def cb_chat_boost_removed(self, update, context):
        self.test_flag = (
            isinstance(context, CallbackContext)
            and isinstance(update.removed_chat_boost, ChatBoostRemoved)
            and not isinstance(update.chat_boost, ChatBoostUpdated)
        )

    async def cb_chat_boost_any(self, update, context):
        self.test_flag = isinstance(context, CallbackContext) and (
            isinstance(update.removed_chat_boost, ChatBoostRemoved)
            or isinstance(update.chat_boost, ChatBoostUpdated)
        )

    @pytest.mark.parametrize(
        argnames=("allowed_types", "cb", "expected"),
        argvalues=[
            (ChatBoostHandler.CHAT_BOOST, "cb_chat_boost_updated", (True, False)),
            (ChatBoostHandler.REMOVED_CHAT_BOOST, "cb_chat_boost_removed", (False, True)),
            (ChatBoostHandler.ANY_CHAT_BOOST, "cb_chat_boost_any", (True, True)),
        ],
        ids=["CHAT_BOOST", "REMOVED_CHAT_BOOST", "ANY_CHAT_MEMBER"],
    )
    async def test_chat_boost_types(self, app, cb, expected, allowed_types):
        result_1, result_2 = expected

        update_type, other = chat_boost_updated_update(), removed_chat_boost_update()

        handler = ChatBoostHandler(getattr(self, cb), chat_boost_types=allowed_types)
        app.add_handler(handler)

        async with app:
            assert handler.check_update(update_type) == result_1
            await app.process_update(update_type)
            assert self.test_flag == result_1

            self.test_flag = False

            assert handler.check_update(other) == result_2
            await app.process_update(other)
            assert self.test_flag == result_2

    def test_other_update_types(self, false_update):
        handler = ChatBoostHandler(self.cb_chat_boost_removed)
        assert not handler.check_update(false_update)
        assert not handler.check_update(True)

    async def test_context(self, app):
        handler = ChatBoostHandler(self.cb_chat_boost_updated)
        app.add_handler(handler)

        async with app:
            await app.process_update(chat_boost_updated_update())
            assert self.test_flag

    def test_with_chat_id(self):
        update = chat_boost_updated_update()
        cb = self.cb_chat_boost_updated
        handler = ChatBoostHandler(cb, chat_id=1)
        assert handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_id=[1])
        assert handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_id=2, chat_username="@chat")
        assert handler.check_update(update)

        handler = ChatBoostHandler(cb, chat_id=2)
        assert not handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_id=[2])
        assert not handler.check_update(update)

    def test_with_username(self):
        update = removed_chat_boost_update()
        cb = self.cb_chat_boost_removed
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username="chat")
        assert handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username="@chat")
        assert handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username=["chat"])
        assert handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username=["@chat"])
        assert handler.check_update(update)
        handler = ChatBoostHandler(
            cb, chat_boost_types=0, chat_id=1, chat_username="@chat_something"
        )
        assert handler.check_update(update)

        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username="chat_b")
        assert not handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username="@chat_b")
        assert not handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username=["chat_b"])
        assert not handler.check_update(update)
        handler = ChatBoostHandler(cb, chat_boost_types=0, chat_username=["@chat_b"])
        assert not handler.check_update(update)

        update.removed_chat_boost.chat._unfreeze()
        update.removed_chat_boost.chat.username = None
        assert not handler.check_update(update)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the InlineQueryHandler class."""

import re
from re import Match, Pattern
from typing import TYPE_CHECKING, Any, TypeVar, cast

from telegram import Update
from telegram._utils.defaultvalue import DEFAULT_TRUE
from telegram._utils.types import DVType
from telegram.ext._handlers.basehandler import BaseHandler
from telegram.ext._utils.types import CCT, HandlerCallback

if TYPE_CHECKING:
    from telegram.ext import Application

RT = TypeVar("RT")


class InlineQueryHandler(BaseHandler[Update, CCT, RT]):
    """
    BaseHandler class to handle Telegram updates that contain a
    :attr:`telegram.Update.inline_query`.
    Optionally based on a regex. Read the documentation of the :mod:`re` module for more
    information.

    Warning:
        * When setting :paramref:`block` to :obj:`False`, you cannot rely on adding custom
          attributes to :class:`telegram.ext.CallbackContext`. See its docs for more info.
        * :attr:`telegram.InlineQuery.chat_type` will not be set for inline queries from secret
          chats and may not be set for inline queries coming from third-party clients. These
          updates won't be handled, if :attr:`chat_types` is passed.

    Examples:
        :any:`Inline Bot <examples.inlinebot>`


    Args:
        callback (:term:`coroutine function`): The callback function for this handler. Will be
            called when :meth:`check_update` has determined that an update should be processed by
            this handler. Callback signature::

                async def callback(update: Update, context: CallbackContext)

            The return value of the callback is usually ignored except for the special case of
            :class:`telegram.ext.ConversationHandler`.
        pattern (:obj:`str` | :func:`re.Pattern <re.compile>`, optional): Regex pattern.
            If not :obj:`None`, :func:`re.match` is used on :attr:`telegram.InlineQuery.query`
            to determine if an update should be handled by this handler.
        block (:obj:`bool`, optional): Determines whether the return value of the callback should
            be awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`. Defaults to :obj:`True`.

            .. seealso:: :wiki:`Concurrency`
        chat_types (list[:obj:`str`], optional): List of allowed chat types. If passed, will only
            handle inline queries with the appropriate :attr:`telegram.InlineQuery.chat_type`.

            .. versionadded:: 13.5
    Attributes:
        callback (:term:`coroutine function`): The callback function for this handler.
        pattern (:obj:`str` | :func:`re.Pattern <re.compile>`): Optional. Regex pattern to test
            :attr:`telegram.InlineQuery.query` against.
        chat_types (list[:obj:`str`]): Optional. List of allowed chat types.

            .. versionadded:: 13.5
        block (:obj:`bool`): Determines whether the return value of the callback should be
            awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`.

    """

    __slots__ = ("chat_types", "pattern")

    def __init__(
        self: "InlineQueryHandler[CCT, RT]",
        callback: HandlerCallback[Update, CCT, RT],
        pattern: str | Pattern[str] | None = None,
        block: DVType[bool] = DEFAULT_TRUE,
        chat_types: list[str] | None = None,
    ):
        super().__init__(callback, block=block)

        if isinstance(pattern, str):
            pattern = re.compile(pattern)

        self.pattern: str | Pattern[str] | None = pattern
        self.chat_types: list[str] | None = chat_types

    def check_update(self, update: object) -> bool | Match[str] | None:
        """
        Determines whether an update should be passed to this handler's :attr:`callback`.

        Args:
            update (:class:`telegram.Update` | :obj:`object`): Incoming update.

        Returns:
            :obj:`bool` | :obj:`re.match`

        """
        if isinstance(update, Update) and update.inline_query:
            if (self.chat_types is not None) and (
                update.inline_query.chat_type not in self.chat_types
            ):
                return False
            if self.pattern and (match := re.match(self.pattern, update.inline_query.query)):
                return match
            if not self.pattern:
                return True
        return None

    def collect_additional_context(
        self,
        context: CCT,
        update: Update,  # noqa: ARG002
        application: "Application[Any, CCT, Any, Any, Any, Any]",  # noqa: ARG002
        check_result: bool | Match[str] | None,
    ) -> None:
        """Add the result of ``re.match(pattern, update.inline_query.query)`` to
        :attr:`CallbackContext.matches` as list with one element.
        """
        if self.pattern:
            check_result = cast("Match", check_result)
            context.matches = [check_result]



#!/usr/bin/env python
# pylint: disable=too-many-arguments
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram InlineQuery."""

from collections.abc import Callable, Sequence
from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._files.location import Location
from telegram._inline.inlinequeryresultsbutton import InlineQueryResultsButton
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.types import JSONDict, ODVInput, TimePeriod

if TYPE_CHECKING:
    from telegram import Bot, InlineQueryResult


class InlineQuery(TelegramObject):
    """
    This object represents an incoming inline query. When the user sends an empty query, your bot
    could return some default or trending results.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`id` is equal.

    .. figure:: https://core.telegram.org/file/464001466/10e4a/r4FKyQ7gw5g.134366/f2\
        606a53d683374703
        :align: center

        Inline queries on Telegram

    .. seealso::
        The :class:`telegram.InlineQueryResult` classes represent the media the user can choose
        from (see above figure).

    Note:
        In Python :keyword:`from` is a reserved word. Use :paramref:`from_user` instead.

    .. versionchanged:: 20.0
        The following are now keyword-only arguments in Bot methods:
        ``{read, write, connect, pool}_timeout``, :paramref:`answer.api_kwargs`,
        ``auto_pagination``. Use a named argument for those,
        and notice that some positional arguments changed position as a result.

    .. versionchanged:: 22.0
        Removed constants ``MIN_START_PARAMETER_LENGTH`` and ``MAX_START_PARAMETER_LENGTH``.
        Use :attr:`telegram.constants.InlineQueryResultsButtonLimit.MIN_START_PARAMETER_LENGTH` and
        :attr:`telegram.constants.InlineQueryResultsButtonLimit.MAX_START_PARAMETER_LENGTH`
        instead.

    Args:
        id (:obj:`str`): Unique identifier for this query.
        from_user (:class:`telegram.User`): Sender.
        query (:obj:`str`): Text of the query (up to
            :tg-const:`telegram.InlineQuery.MAX_QUERY_LENGTH` characters).
        offset (:obj:`str`): Offset of the results to be returned, can be controlled by the bot.
        chat_type (:obj:`str`, optional): Type of the chat, from which the inline query was sent.
            Can be either :tg-const:`telegram.Chat.SENDER` for a private chat with the inline query
            sender, :tg-const:`telegram.Chat.PRIVATE`, :tg-const:`telegram.Chat.GROUP`,
            :tg-const:`telegram.Chat.SUPERGROUP` or :tg-const:`telegram.Chat.CHANNEL`. The chat
            type should be always known for requests sent from official clients and most
            third-party clients, unless the request was sent from a secret chat.

            .. versionadded:: 13.5
        location (:class:`telegram.Location`, optional): Sender location, only for bots that
            request user location.

    Attributes:
        id (:obj:`str`): Unique identifier for this query.
        from_user (:class:`telegram.User`): Sender.
        query (:obj:`str`): Text of the query (up to
            :tg-const:`telegram.InlineQuery.MAX_QUERY_LENGTH` characters).
        offset (:obj:`str`): Offset of the results to be returned, can be controlled by the bot.
        chat_type (:obj:`str`): Optional. Type of the chat, from which the inline query was sent.
            Can be either :tg-const:`telegram.Chat.SENDER` for a private chat with the inline query
            sender, :tg-const:`telegram.Chat.PRIVATE`, :tg-const:`telegram.Chat.GROUP`,
            :tg-const:`telegram.Chat.SUPERGROUP` or :tg-const:`telegram.Chat.CHANNEL`. The chat
            type should be always known for requests sent from official clients and most
            third-party clients, unless the request was sent from a secret chat.

            .. versionadded:: 13.5
        location (:class:`telegram.Location`): Optional. Sender location, only for bots that
            request user location.

    """

    __slots__ = ("chat_type", "from_user", "id", "location", "offset", "query")

    def __init__(
        self,
        id: str,  # pylint: disable=redefined-builtin
        from_user: User,
        query: str,
        offset: str,
        location: "Location | None" = None,
        chat_type: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.id: str = id
        self.from_user: User = from_user
        self.query: str = query
        self.offset: str = offset

        # Optional
        self.location: Location | None = location
        self.chat_type: str | None = chat_type

        self._id_attrs = (self.id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "InlineQuery":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["from_user"] = de_json_optional(data.pop("from", None), User, bot)
        data["location"] = de_json_optional(data.get("location"), Location, bot)

        return super().de_json(data=data, bot=bot)

    async def answer(
        self,
        results: (
            Sequence["InlineQueryResult"] | Callable[[int], Sequence["InlineQueryResult"] | None]
        ),
        cache_time: TimePeriod | None = None,
        is_personal: bool | None = None,
        next_offset: str | None = None,
        button: InlineQueryResultsButton | None = None,
        *,
        current_offset: str | None = None,
        auto_pagination: bool = False,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> bool:
        """Shortcut for::

            await bot.answer_inline_query(
                update.inline_query.id,
                *args,
                current_offset=self.offset if auto_pagination else None,
                **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.answer_inline_query`.

        .. versionchanged:: 20.0
            Raises :class:`ValueError` instead of :class:`TypeError`.

        Keyword Args:
            auto_pagination (:obj:`bool`, optional): If set to :obj:`True`, :attr:`offset` will be
                passed as
                :paramref:`current_offset <telegram.Bot.answer_inline_query.current_offset>` to
                :meth:`telegram.Bot.answer_inline_query`.
                Defaults to :obj:`False`.

        Raises:
            ValueError: If both :paramref:`~telegram.Bot.answer_inline_query.current_offset` and
                :paramref:`auto_pagination` are supplied.
        """
        if current_offset and auto_pagination:
            raise ValueError("current_offset and auto_pagination are mutually exclusive!")
        return await self.get_bot().answer_inline_query(
            inline_query_id=self.id,
            current_offset=self.offset if auto_pagination else current_offset,
            results=results,
            cache_time=cache_time,
            is_personal=is_personal,
            next_offset=next_offset,
            button=button,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    MAX_RESULTS: Final[int] = constants.InlineQueryLimit.RESULTS
    """:const:`telegram.constants.InlineQueryLimit.RESULTS`

    .. versionadded:: 13.2
    """
    MAX_OFFSET_LENGTH: Final[int] = constants.InlineQueryLimit.MAX_OFFSET_LENGTH
    """:const:`telegram.constants.InlineQueryLimit.MAX_OFFSET_LENGTH`

    .. versionadded:: 20.0
    """
    MAX_QUERY_LENGTH: Final[int] = constants.InlineQueryLimit.MAX_QUERY_LENGTH
    """:const:`telegram.constants.InlineQueryLimit.MAX_QUERY_LENGTH`

    .. versionadded:: 20.0
    """



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the BusinessMessagesDeletedHandler class."""

from typing import TypeVar

from telegram import Update
from telegram._utils.defaultvalue import DEFAULT_TRUE
from telegram._utils.types import SCT, DVType
from telegram.ext._handlers.basehandler import BaseHandler
from telegram.ext._utils._update_parsing import parse_chat_id, parse_username
from telegram.ext._utils.types import CCT, HandlerCallback

RT = TypeVar("RT")


class BusinessMessagesDeletedHandler(BaseHandler[Update, CCT, RT]):
    """Handler class to handle
    :attr:`deleted Telegram Business messages <telegram.Update.deleted_business_messages>`.

    .. versionadded:: 21.1

    Args:
        callback (:term:`coroutine function`): The callback function for this handler. Will be
            called when :meth:`check_update` has determined that an update should be processed by
            this handler. Callback signature::

                async def callback(update: Update, context: CallbackContext)
        chat_id (:obj:`int` | Collection[:obj:`int`], optional): Filters requests to allow only
            those which are from the specified chat ID(s).

        username (:obj:`str` | Collection[:obj:`str`], optional): Filters requests to allow only
            those which are from the specified username(s).

        block (:obj:`bool`, optional): Determines whether the return value of the callback should
            be awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`. Defaults to :obj:`True`.

            .. seealso:: :wiki:`Concurrency`
    Attributes:
        callback (:term:`coroutine function`): The callback function for this handler.
        block (:obj:`bool`): Determines whether the return value of the callback should be
            awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`.
    """

    __slots__ = (
        "_chat_ids",
        "_usernames",
    )

    def __init__(
        self: "BusinessMessagesDeletedHandler[CCT, RT]",
        callback: HandlerCallback[Update, CCT, RT],
        chat_id: SCT[int] | None = None,
        username: SCT[str] | None = None,
        block: DVType[bool] = DEFAULT_TRUE,
    ):
        super().__init__(callback, block=block)

        self._chat_ids = parse_chat_id(chat_id)
        self._usernames = parse_username(username)

    def check_update(self, update: object) -> bool:
        """Determines whether an update should be passed to this handler's :attr:`callback`.

        Args:
            update (:class:`telegram.Update` | :obj:`object`): Incoming update.

        Returns:
            :obj:`bool`

        """
        if isinstance(update, Update) and update.deleted_business_messages:
            if not self._chat_ids and not self._usernames:
                return True
            if update.deleted_business_messages.chat.id in self._chat_ids:
                return True
            return update.deleted_business_messages.chat.username in self._usernames
        return False



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import InputTextMessageContent, LinkPreviewOptions, MessageEntity
from telegram.constants import ParseMode
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def input_text_message_content():
    return InputTextMessageContent(
        InputTextMessageContentTestBase.message_text,
        parse_mode=InputTextMessageContentTestBase.parse_mode,
        entities=InputTextMessageContentTestBase.entities,
        link_preview_options=InputTextMessageContentTestBase.link_preview_options,
    )


class InputTextMessageContentTestBase:
    message_text = "*message text*"
    parse_mode = ParseMode.MARKDOWN
    entities = [MessageEntity(MessageEntity.ITALIC, 0, 7)]
    disable_web_page_preview = False
    link_preview_options = LinkPreviewOptions(False, url="https://python-telegram-bot.org")


class TestInputTextMessageContentWithoutRequest(InputTextMessageContentTestBase):
    def test_slot_behaviour(self, input_text_message_content):
        inst = input_text_message_content
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, input_text_message_content):
        assert input_text_message_content.parse_mode == self.parse_mode
        assert input_text_message_content.message_text == self.message_text
        assert input_text_message_content.entities == tuple(self.entities)
        assert input_text_message_content.link_preview_options == self.link_preview_options

    def test_entities_always_tuple(self):
        input_text_message_content = InputTextMessageContent("text")
        assert input_text_message_content.entities == ()

    def test_to_dict(self, input_text_message_content):
        input_text_message_content_dict = input_text_message_content.to_dict()

        assert isinstance(input_text_message_content_dict, dict)
        assert (
            input_text_message_content_dict["message_text"]
            == input_text_message_content.message_text
        )
        assert (
            input_text_message_content_dict["parse_mode"] == input_text_message_content.parse_mode
        )
        assert input_text_message_content_dict["entities"] == [
            ce.to_dict() for ce in input_text_message_content.entities
        ]
        assert (
            input_text_message_content_dict["link_preview_options"]
            == input_text_message_content.link_preview_options.to_dict()
        )

    def test_equality(self):
        a = InputTextMessageContent("text")
        b = InputTextMessageContent("text", parse_mode=ParseMode.HTML)
        c = InputTextMessageContent("label")
        d = ParseMode.HTML

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

    def test_mutually_exclusive(self):
        with pytest.raises(ValueError, match="`link_preview_options` are mutually exclusive"):
            InputTextMessageContent(
                "text", disable_web_page_preview=True, link_preview_options=LinkPreviewOptions()
            )

    def test_disable_web_page_preview_deprecation(self):
        itmc = InputTextMessageContent("text", disable_web_page_preview=True)
        assert itmc.link_preview_options.is_disabled is True



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import asyncio
import os
import sys
import zoneinfo
from pathlib import Path
from uuid import uuid4

import pytest

from telegram import (
    CallbackQuery,
    Chat,
    ChosenInlineResult,
    InlineQuery,
    Message,
    PreCheckoutQuery,
    ShippingQuery,
    Update,
    User,
)
from telegram.ext import Defaults
from tests.auxil.build_messages import DATE, make_message
from tests.auxil.ci_bots import BOT_INFO_PROVIDER, JOB_INDEX
from tests.auxil.constants import PRIVATE_KEY, TEST_TOPIC_ICON_COLOR, TEST_TOPIC_NAME
from tests.auxil.envvars import GITHUB_ACTIONS, TEST_WITH_OPT_DEPS, env_var_2_bool
from tests.auxil.files import data_file
from tests.auxil.networking import NonchalantHttpxRequest
from tests.auxil.pytest_classes import PytestBot, make_bot

if TEST_WITH_OPT_DEPS:
    import pytz


# This is here instead of in setup.cfg due to https://github.com/pytest-dev/pytest/issues/8343
def pytest_runtestloop(session: pytest.Session):
    session.add_marker(
        pytest.mark.filterwarnings("ignore::telegram.warnings.PTBDeprecationWarning")
    )


def no_rerun_after_xfail_or_flood(error, name, test: pytest.Function, plugin):
    """Don't rerun tests that have xfailed when marked with xfail, or when we hit a flood limit."""
    xfail_present = test.get_closest_marker(name="xfail")
    if getattr(error[1], "msg", "") is None:
        raise error[1]
    did_we_flood = "flood" in getattr(error[1], "msg", "")  # _pytest.outcomes.XFailed has 'msg'
    return not (xfail_present or did_we_flood)


def pytest_collection_modifyitems(items: list[pytest.Item]):
    """Here we add a flaky marker to all request making tests and a (no_)req marker to the rest."""
    for item in items:  # items are the test methods
        parent = item.parent  # Get the parent of the item (class, or module if defined outside)
        if parent is None:  # should never happen, but just in case
            return
        if (  # Check if the class name ends with 'WithRequest' and if it has no flaky marker
            parent.name.endswith("WithRequest")
            and not parent.get_closest_marker(  # get_closest_marker gets pytest.marks with `name`
                name="flaky"
            )  # don't add/override any previously set markers
            and not parent.get_closest_marker(name="req")
        ):  # Add the flaky marker with a rerun filter to the class
            parent.add_marker(pytest.mark.flaky(3, 1, rerun_filter=no_rerun_after_xfail_or_flood))
            parent.add_marker(pytest.mark.req)
        # Add the no_req marker to all classes that end with 'WithoutRequest' and don't have it
        elif parent.name.endswith("WithoutRequest") and not parent.get_closest_marker(
            name="no_req"
        ):
            parent.add_marker(pytest.mark.no_req)


if GITHUB_ACTIONS and JOB_INDEX == 0:
    # let's not slow down the tests too much with these additional checks
    # that's why we run them only in GitHub actions and only on *one* of the several test
    # matrix entries
    @pytest.fixture(autouse=True)
    def _disallow_requests_in_without_request_tests(request):
        """This fixture prevents tests that don't require requests from using the online-bot.
        This is a sane-effort approach on trying to prevent requests from being made in the
        *WithoutRequest classes. Note that we can not prevent all requests, as one can still
        manually build a `Bot` object or use `httpx` directly. See #4317 and #4465 for some
        discussion.
        """

        if type(request).__name__ == "SubRequest":
            # Some fixtures used in the *WithoutRequests test classes do use requests, e.g.
            # `animation`. Separating that would be too much effort, hence we allow that.
            # Unfortunately the `SubRequest` class is not public, so we check only the name for
            # less dependency on pytest's internal structure.
            return

        if not request.cls:
            return
        name = request.cls.__name__
        if not name.endswith("WithoutRequest") or not request.fixturenames:
            return

        if "bot" in request.fixturenames:
            pytest.fail(
                f"Test function {request.function} in test class {name} should not have a `bot` "
                f"fixture. Use `offline_bot` instead."
            )


@pytest.fixture(scope="module", params=["true", "1", "false", "gibberish", None])
def PTB_TIMEDELTA(request):
    # Here we manually use monkeypatch to give this fixture module scope
    monkeypatch = pytest.MonkeyPatch()
    if request.param is not None:
        monkeypatch.setenv("PTB_TIMEDELTA", request.param)
    else:
        monkeypatch.delenv("PTB_TIMEDELTA", raising=False)
    yield env_var_2_bool(os.getenv("PTB_TIMEDELTA"))
    monkeypatch.undo()


# Redefine the event_loop fixture to have a session scope. Otherwise `bot` fixture can't be
# session. See https://github.com/pytest-dev/pytest-asyncio/issues/68 for more details.
@pytest.fixture(scope="session")
def event_loop(request):
    # ever since ProactorEventLoop became the default in Win 3.8+, the app crashes after the loop
    # is closed. Hence, we use SelectorEventLoop on Windows to avoid this. See
    # https://github.com/python/cpython/issues/83413, https://github.com/encode/httpx/issues/914
    if sys.platform.startswith("win"):
        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
    return asyncio.get_event_loop_policy().new_event_loop()
    # loop.close() # instead of closing here, do that at the every end of the test session


@pytest.fixture(scope="session")
def bot_info() -> dict[str, str]:
    return BOT_INFO_PROVIDER.get_info()


@pytest.fixture(scope="session")
async def bot(bot_info):
    """Makes an ExtBot instance with the given bot_info"""
    async with make_bot(bot_info, offline=False) as _bot:
        yield _bot


@pytest.fixture(scope="session")
async def offline_bot(bot_info):
    """Makes an offline Bot instance with the given bot_info
    Note that in tests/ext we also override the `bot` fixture to return the offline bot instead.
    """
    async with make_bot(bot_info, offline=True) as _bot:
        yield _bot


@pytest.fixture
def one_time_bot(bot_info):
    """A function scoped bot since the session bot would shutdown when `async with app` finishes"""
    return make_bot(bot_info, offline=False)


@pytest.fixture(scope="session")
async def cdc_bot(bot_info):
    """Makes an ExtBot instance with the given bot_info that uses arbitrary callback_data"""
    async with make_bot(bot_info, arbitrary_callback_data=True, offline=False) as _bot:
        yield _bot


@pytest.fixture(scope="session")
async def raw_bot(bot_info):
    """Makes an regular Bot instance with the given bot_info"""
    async with PytestBot(
        bot_info["token"],
        private_key=PRIVATE_KEY if TEST_WITH_OPT_DEPS else None,
        request=NonchalantHttpxRequest(8),
        get_updates_request=NonchalantHttpxRequest(1),
    ) as _bot:
        yield _bot


# Here we store the default bots so that we don't have to create them again and again.
# They are initialized but not shutdown on pytest_sessionfinish because it is causing
# problems with the event loop (Event loop is closed).
_default_bots = {}


@pytest.fixture(scope="session")
async def default_bot(request, bot_info):
    param = request.param if hasattr(request, "param") else {}
    defaults = Defaults(**param)

    # If the bot is already created, return it. Else make a new one.
    default_bot = _default_bots.get(defaults)
    if default_bot is None:
        default_bot = make_bot(bot_info, defaults=defaults, offline=False)
        await default_bot.initialize()
        _default_bots[defaults] = default_bot  # Defaults object is hashable
    return default_bot


@pytest.fixture(scope="session")
async def tz_bot(timezone, bot_info):
    defaults = Defaults(tzinfo=timezone)
    try:  # If the bot is already created, return it. Saves time since get_me is not called again.
        return _default_bots[defaults]
    except KeyError:
        default_bot = make_bot(bot_info, defaults=defaults, offline=False)
        await default_bot.initialize()
        _default_bots[defaults] = default_bot
        return default_bot


@pytest.fixture(scope="session")
def chat_id(bot_info):
    return bot_info["chat_id"]


@pytest.fixture(scope="session")
def super_group_id(bot_info):
    return bot_info["super_group_id"]


@pytest.fixture(scope="session")
def forum_group_id(bot_info):
    return int(bot_info["forum_group_id"])


@pytest.fixture(scope="session")
def channel_id(bot_info):
    return bot_info["channel_id"]


@pytest.fixture(scope="session")
def provider_token(bot_info):
    return bot_info["payment_provider_token"]


@pytest.fixture(scope="session")
def subscription_channel_id(bot_info):
    return bot_info["subscription_channel_id"]


@pytest.fixture
def thumb_file():
    with data_file("thumb.jpg").open("rb") as f:
        yield f


@pytest.fixture(scope="module")
def class_thumb_file():
    with data_file("thumb.jpg").open("rb") as f:
        yield f


@pytest.fixture(scope="session")
async def emoji_id(bot):
    emoji_sticker_list = await bot.get_forum_topic_icon_stickers()
    first_sticker = emoji_sticker_list[0]
    return first_sticker.custom_emoji_id


@pytest.fixture
async def real_topic(bot, emoji_id, forum_group_id):
    result = await bot.create_forum_topic(
        chat_id=forum_group_id,
        name=TEST_TOPIC_NAME,
        icon_color=TEST_TOPIC_ICON_COLOR,
        icon_custom_emoji_id=emoji_id,
    )

    yield result

    result = await bot.delete_forum_topic(
        chat_id=forum_group_id, message_thread_id=result.message_thread_id
    )
    assert result is True, "Topic was not deleted"


def _get_false_update_fixture_decorator_params():
    message = Message(1, DATE, Chat(1, ""), from_user=User(1, "", False), text="test")
    params = [
        {"callback_query": CallbackQuery(1, User(1, "", False), "chat", message=message)},
        {"channel_post": message},
        {"edited_channel_post": message},
        {"inline_query": InlineQuery(1, User(1, "", False), "", "")},
        {"chosen_inline_result": ChosenInlineResult("id", User(1, "", False), "")},
        {"shipping_query": ShippingQuery("id", User(1, "", False), "", None)},
        {"pre_checkout_query": PreCheckoutQuery("id", User(1, "", False), "", 0, "")},
        {"callback_query": CallbackQuery(1, User(1, "", False), "chat")},
    ]
    ids = tuple(key for kwargs in params for key in kwargs)
    return {"params": params, "ids": ids}


@pytest.fixture(**_get_false_update_fixture_decorator_params())
def false_update(request):
    return Update(update_id=1, **request.param)


@pytest.fixture(
    scope="session",
    params=[pytz.timezone, zoneinfo.ZoneInfo] if TEST_WITH_OPT_DEPS else [zoneinfo.ZoneInfo],
)
def _tz_implementation(request):
    # This fixture is used to parametrize the timezone fixture
    # This is similar to what @pyttest.mark.parametrize does but for fixtures
    # However, this is needed only internally for the `tzinfo` fixture, so we keep it private
    return request.param


@pytest.fixture(scope="session", params=["Europe/Berlin", "Asia/Singapore", "UTC"])
def tzinfo(request, _tz_implementation):
    return _tz_implementation(request.param)


@pytest.fixture(scope="session")
def timezone(tzinfo):
    return tzinfo


@pytest.fixture
def tmp_file(tmp_path) -> Path:
    return tmp_path / uuid4().hex


@pytest.fixture(scope="session")
def dummy_message():
    return make_message("dummy_message")


@pytest.fixture(scope="session")
def dummy_message_dict(dummy_message):
    return dummy_message.to_dict()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the LinkPreviewOptions class."""

from telegram._telegramobject import TelegramObject
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.types import JSONDict, ODVInput


class LinkPreviewOptions(TelegramObject):
    """
    Describes the options used for link preview generation.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`is_disabled`, :attr:`url`, :attr:`prefer_small_media`,
    :attr:`prefer_large_media`, and :attr:`show_above_text` are equal.

    .. versionadded:: 20.8

    Args:
        is_disabled (:obj:`bool`, optional): :obj:`True`, if the link preview is disabled.
        url (:obj:`str`, optional): The URL to use for the link preview. If empty, then the first
            URL found in the message text will be used.
        prefer_small_media (:obj:`bool`, optional): :obj:`True`, if the media in the link preview
            is supposed to be shrunk; ignored if the URL isn't explicitly specified or media size
            change isn't supported for the preview.
        prefer_large_media (:obj:`bool`, optional): :obj:`True`, if the media in the link preview
            is supposed to be enlarged; ignored if the URL isn't explicitly specified or media
            size change isn't supported for the preview.
        show_above_text (:obj:`bool`, optional): :obj:`True`, if the link preview must be shown
            above the message text; otherwise, the link preview will be shown below the message
            text.

    Attributes:
        is_disabled (:obj:`bool`): Optional. :obj:`True`, if the link preview is disabled.
        url (:obj:`str`): Optional. The URL to use for the link preview. If empty, then the first
            URL found in the message text will be used.
        prefer_small_media (:obj:`bool`): Optional. :obj:`True`, if the media in the link preview
            is supposed to be shrunk; ignored if the URL isn't explicitly specified or media size
            change isn't supported for the preview.
        prefer_large_media (:obj:`bool`): Optional. :obj:`True`, if the media in the link preview
            is supposed to be enlarged; ignored if the URL isn't explicitly specified or media size
            change isn't supported for the preview.
        show_above_text (:obj:`bool`): Optional. :obj:`True`, if the link preview must be shown
            above the message text; otherwise, the link preview will be shown below the message
            text.
    """

    __slots__ = (
        "is_disabled",
        "prefer_large_media",
        "prefer_small_media",
        "show_above_text",
        "url",
    )

    def __init__(
        self,
        is_disabled: ODVInput[bool] = DEFAULT_NONE,
        url: ODVInput[str] = DEFAULT_NONE,
        prefer_small_media: ODVInput[bool] = DEFAULT_NONE,
        prefer_large_media: ODVInput[bool] = DEFAULT_NONE,
        show_above_text: ODVInput[bool] = DEFAULT_NONE,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        # Optionals

        self.is_disabled: ODVInput[bool] = is_disabled
        self.url: ODVInput[str] = url
        self.prefer_small_media: ODVInput[bool] = prefer_small_media
        self.prefer_large_media: ODVInput[bool] = prefer_large_media
        self.show_above_text: ODVInput[bool] = show_above_text

        self._id_attrs = (
            self.is_disabled,
            self.url,
            self.prefer_small_media,
            self.prefer_large_media,
            self.show_above_text,
        )
        self._freeze()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import BotCommand, ChatOwnerChanged, ChatOwnerLeft, User
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def chat_owner_changed():
    return ChatOwnerChanged(ChatOwnerTestBase.new_owner)


@pytest.fixture(scope="module")
def chat_owner_left():
    return ChatOwnerLeft(ChatOwnerTestBase.new_owner)


class ChatOwnerTestBase:
    new_owner = User(1, "SnowCrash", False)


class TestChatOwnerChangedWithoutRequest(ChatOwnerTestBase):
    def test_slot_behaviour(self, chat_owner_changed):
        inst = chat_owner_changed
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "new_owner": self.new_owner.to_dict(),
        }
        chat_owner_changed = ChatOwnerChanged.de_json(json_dict, offline_bot)
        assert chat_owner_changed.api_kwargs == {}

        assert chat_owner_changed.new_owner == self.new_owner
        assert chat_owner_changed.new_owner.first_name == self.new_owner.first_name

    def test_to_dict(self, chat_owner_changed):
        chat_owner_changed_dict = chat_owner_changed.to_dict()

        assert isinstance(chat_owner_changed_dict, dict)
        assert chat_owner_changed_dict["new_owner"] == chat_owner_changed.new_owner.to_dict()

    def test_equality(self, chat_owner_changed):
        a = chat_owner_changed
        b = ChatOwnerChanged(ChatOwnerTestBase.new_owner)
        c = ChatOwnerChanged(User(2, "SnowCrash", False))
        d = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


class TestChatOwnerLeftWithoutRequest(ChatOwnerTestBase):
    def test_slot_behaviour(self, chat_owner_left):
        inst = chat_owner_left
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "new_owner": self.new_owner.to_dict(),
        }
        chat_owner_left = ChatOwnerLeft.de_json(json_dict, offline_bot)
        assert chat_owner_left.api_kwargs == {}

        assert chat_owner_left.new_owner == self.new_owner
        assert chat_owner_left.new_owner.first_name == self.new_owner.first_name

    def test_to_dict(self, chat_owner_left):
        chat_owner_left_dict = chat_owner_left.to_dict()

        assert isinstance(chat_owner_left_dict, dict)
        assert chat_owner_left_dict["new_owner"] == chat_owner_left.new_owner.to_dict()

    def test_equality(self, chat_owner_left):
        a = chat_owner_left
        b = ChatOwnerLeft(ChatOwnerTestBase.new_owner)
        c = ChatOwnerLeft(User(2, "SnowCrash", False))
        d = ChatOwnerLeft()
        e = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram Birthday."""

import datetime as dtm

from telegram._telegramobject import TelegramObject
from telegram._utils.types import JSONDict


class Birthdate(TelegramObject):
    """
    This object describes the birthdate of a user.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`day`, and :attr:`month` are equal.

    .. versionadded:: 21.1

    Args:
        day (:obj:`int`): Day of the user's birth; 1-31.
        month (:obj:`int`): Month of the user's birth; 1-12.
        year (:obj:`int`, optional): Year of the user's birth.

    Attributes:
        day (:obj:`int`): Day of the user's birth; 1-31.
        month (:obj:`int`): Month of the user's birth; 1-12.
        year (:obj:`int`): Optional. Year of the user's birth.

    """

    __slots__ = ("day", "month", "year")

    def __init__(
        self,
        day: int,
        month: int,
        year: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        # Required
        self.day: int = day
        self.month: int = month
        # Optional
        self.year: int | None = year

        self._id_attrs = (
            self.day,
            self.month,
        )

        self._freeze()

    def to_date(self, year: int | None = None) -> dtm.date:
        """Return the birthdate as a date object.

        .. versionchanged:: 21.2
           Now returns a :obj:`datetime.date` object instead of a :obj:`datetime.datetime` object,
           as was originally intended.

        Args:
            year (:obj:`int`, optional): The year to use. Required, if the :attr:`year` was not
                present.

        Returns:
            :obj:`datetime.date`: The birthdate as a date object.
        """
        if self.year is None and year is None:
            raise ValueError(
                "The `year` argument is required if the `year` attribute was not present."
            )

        return dtm.date(year or self.year, self.month, self.day)  # type: ignore[arg-type]
